@@ -1,30 +1,25 @@
 =head1 NAME
 
-Xacobeo::Document - An XML document and it's related information.
+Xacobeo::Document - XML document and it's related information
 
 =head1 SYNOPSIS
 
 	use Xacobeo::Document;
 	
-	my $document = Xacobeo::Document->new('file.xml');
+	my $document Xacobeo::Document->new('file.xml');
 	
 	my $namespaces = $document->namespaces(); # Hashref
-	while (my ($uri, $prefix) = each %{ $namespaces }) {
+	while (my ($prefix, $uri) = each %{ $namespaces }) {
 		printf "%-5s: %s\n", $prefix, $uri;
 	}
 	
-	
-	my $nodes = $document->find('/x:html//x:a[@href]');
-	foreach my $node ($nodes->get_nodelist) {
-		print "Got node ", $node->name, "\n";
-	}
-	
+	my @nodes = $document->findnodes('/x:html//x:a[@href]');
 	$document->validate('/x:html//x:a[@href]') or die "Invalid XPath expression";
 
 =head1 DESCRIPTION
 
-This package wraps an XML document with it's corresponding meta information
-(namespaces, source, etc).
+This package wraps an XML document with it's corresponding meta information (
+namespaces, source, etc).
 
 =head1 METHODS
 
@@ -38,12 +33,11 @@ use strict;
 use warnings;
 
 use XML::LibXML;
+use UNIVERSAL qw(isa);
+
 use Data::Dumper;
 use Carp;
 
-use Xacobeo::Utils qw(:dom);
-
-
 use base qw(Class::Accessor::Fast);
 __PACKAGE__->mk_accessors(
 	qw(
@@ -58,7 +52,7 @@ __PACKAGE__->mk_accessors(
 
 Creates a new instance.
 
-Parameters:
+Paramters:
 
 	$source: the source of the XML document, this can be a file name.
 
@@ -77,33 +71,31 @@ sub new {
 }
 
 
-=head2 find
+=head2 findnodes
 
-Runs the given XPath query on the document and returns the results. The results
-could be a node list or a single value like a boolean, a number or a scalar if
-an expression is passed.
+Runs the given XPath query on the document. The resutls are returned as a list.
+In scalar context the number of nodes are returned.
 
-This method croaks if the expression can't be evaluated.
+Paramters:
 
-Parameters:
-
-	$xpath: the XPath expression to execute.
+	$xpath: a valid XPath expression.
 
 =cut
 
-sub find {
+sub findnodes {
 	my $self = shift;
 	my ($xpath) = @_;
 	
-	my $result;
+	my @nodes = ();
 	eval {
-		$result = $self->xpath->find($xpath, $self->xml);
+		@nodes = $self->xpath->findnodes($xpath, $self->xml);
 	};
 	if (my $error = $@) {
-		croak $error;
+#		warn "XPath '$xpath' failed: $error";
+		return;
 	}
 	
-	return $result;
+	return wantarray ? @nodes : scalar @nodes;
 }
 
 
@@ -113,12 +105,9 @@ Validates the syntax of the given XPath query. The syntax is validated within a
 context that has the same namespaces as the ones defined in the current XML
 document.
 
-B<NOTE>: This method can't validate if undefined functions or variables are
-used.
+Paramters:
 
-Parameters:
-
-	$xpath: the XPath expression to validate.
+	$xpath: a valid XPath expression.
 
 =cut
 
@@ -143,37 +132,9 @@ sub validate {
 }
 
 
-=head2 get_prefixed_name
-
-Returns the node name by prefixing it with our prefixes in the case where
-namespaces are used.
-
-=cut
-
-sub get_prefixed_name {
-	my $self = shift;
-	my ($node) = @_;
-
-	my $name = $node->localname;
-	my $uri = $node->namespaceURI();
-
-	# Check if the node uses a namespace if so return the name with our prefix
-	if (defined $uri and my $namespace = $self->{namespaces}{$uri}) {
-		return "$namespace:$name";
-	}
-
-	return $name;
-}
-
-
-
-=head2 namespaces
-
-Returns the namespaces declared in the document. The namespaces are returned in
-a hash REF where the URIs are used as a key and the prefix as a value.
-
-=cut
-
+#
+# Get/Set the namespaces.
+#
 sub namespaces {
 	my $self = shift;
 	if (@_) {
@@ -184,7 +145,7 @@ sub namespaces {
 
 
 #
-# Loads the XML document. This method will also find the namespaces used in the
+# Loads the XML document. This method will also find the namespaces used by the
 # document.
 #
 sub _load_document {
@@ -196,6 +157,7 @@ sub _load_document {
 	
 	# Parse the document
 	my $parser = _construct_xml_parser();
+
 	my $xml = $parser->parse_file($source);
 	$self->xml($xml);
 	
@@ -226,47 +188,57 @@ sub _construct_xml_parser {
 #
 # Finds every namespace declared in the document.
 #
-# Each prefix is warrantied to be unique. The function will assign the first
-# prefix seen for each namespace.
+sub _fetch_namespaces {
+	my ($node, $collected) = @_;
+
+	foreach my $child ($node->childNodes) {
+		
+		# Only elements are allowed to declare namespaces
+		next unless $child->isa('XML::LibXML::Element');
+		
+		foreach my $namespace ($child->getNamespaces) {
+			my $uri = $namespace->getData;
+			$collected->{$uri} ||= $namespace->getLocalName;
+		}
+		
+		_fetch_namespaces($child, $collected);
+	}
+}
+
+
 #
-# NOTE: libxml2 assumes that the prefix 'xml' is is bounded to the URI
-#       http://www.w3.org/XML/1998/namespace, therefore this namespace will
-#       always be returned even if it's not declared in the document.
+# Finds every namespace declared in the document.
 #
-# The prefixes are returned in an hash ref of type ($uri => $prefix).
+# Each prefix is warrantied to be unique.
+# The function will assign the first prefix seen for each namespace.
+#
+# The prefixes are returned in an hash ref of type $prefix => $uri.
 #
 sub _get_all_namespaces {
 	my ($node) = @_;
-
+#FIXME the document test/beers.xml uses an empty namespace, this code has problems with it
 	# Find the namespaces ($uri -> $prefix)
-	my %namespaces = (
-		XML_XML_NS() => 'xml',
-	);
-	foreach my $namespace ($node->findnodes('.//namespace::*')) {
-		my $uri = $namespace->getData;
-		$namespaces{$uri} ||= $namespace->getLocalName;
-	}
+	my %namespaces = ();
+	_fetch_namespaces($node, \%namespaces);
 	
 	# Reverse the namespaces ($prefix -> $uri) and make sure that the prefixes
 	# don't clash with each other.
-	my $cleaned = {};
-	my $namespaces = {};
+	my $cleanned = {};
 	my $index = 0;
 	while (my ($uri, $prefix) = each %namespaces) {
 
 		# Make sure that the prefixes are unique
-		if (! defined $prefix or exists $cleaned->{$prefix}) {
+		if (! defined $prefix or exists $cleanned->{$prefix}) {
 			# Assign a new prefix until unique
 			do {
 				$prefix = 'default' . ($index ? $index : '');
 				++$index;
-			} while (exists $cleaned->{$prefix});
+			} while (exists $cleanned->{$prefix});
 		}
-		$cleaned->{$prefix} = $uri;
-		$namespaces->{$uri} = $prefix;
+		$cleanned->{$prefix} = $uri;
 	}
 
-	return $namespaces;
+	return $cleanned;
 }
 
 
@@ -280,7 +252,7 @@ sub _create_xpath_context {
 	my $context = XML::LibXML::XPathContext->new();
 
 	# Add the namespaces to the XPath context
-	while (my ($uri, $prefix) = each %{ $self->namespaces }) {
+	while (my ($prefix, $uri) = each %{ $self->namespaces }) {
 		$context->registerNs($prefix, $uri);
 	}
 	
@@ -290,18 +262,3 @@ sub _create_xpath_context {
 
 # A true value
 1;
-
-=head1 AUTHORS
-
-Emmanuel Rodriguez E<lt>potyl@cpan.orgE<gt>.
-
-=head1 COPYRIGHT AND LICENSE
-
-Copyright (C) 2008 by Emmanuel Rodriguez.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.8 or,
-at your option, any later version of Perl 5 you may have available.
-
-=cut
-
